"""
To replace google.colab.files
Add files.edit(filepath) and %edit
"""
from pathlib import Path
from IPython.core.magic import register_line_magic
from google.colab.files import *  # upload, download, view


@register_line_magic
def edit(filepath):
    """
    This can be used as files.edit(fname) or %edit fname
    """
    path = Path(filepath)
    if not path.exists():
        path.touch()  # create empty
    view(path)



""" make calling autoreload easier """

# from importlib import reload    # probably don't need manual reload
from IPython import get_ipython


magic = get_ipython().run_line_magic
magic('load_ext', 'autoreload')
magic('autoreload', '2')  # on by default


def on():
    magic('autoreload', '2')

def off():
    magic('autoreload', '0')


"""
Open another tab for web console (command line)
"""
import os
from subprocess import Popen, PIPE


# Install
url = "https://github.com/gravitational/teleconsole/releases/download/0.4.0/teleconsole-v0.4.0-linux-amd64.tar.gz"
os.system(f"curl -L {url} | tar xz")  # download & extract
os.system("mv teleconsole /usr/local/bin/")  # in PATH

# Set PS1, directory
with open("/root/.bashrc", "a") as f:
    f.write('PS1="\e[1;36m\w\e[m# "\n')
    f.write("cd /content \n")
    f.write("PATH=/usr/local/nvidia/bin:/usr/local/cuda/bin:/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/tools/node/bin:/tools/google-cloud-sdk/bin:/opt/bin \n")


def start():
    """ start the teleconsole, and print URL """
    process = Popen("teleconsole", shell=True,
                    stdin=PIPE, stdout=PIPE, stderr=PIPE)
    for i in range(6):
        line = process.stdout.readline()
    url = line.decode().strip().split()[-1]
    print("Console URL:", url)


def stop():
    os.system("pkill teleconsole")

def restart():
    stop()
    start()



""" String processing utilities
"""

def ranges_to_list(range_str):
    """ ranges_to_list("1-3,5-7") == [1,2,3,5,6,7] """
    num_list = []
    for rng in range_str.split(','):
        a, b = map(int, rng.split('-'))
        num_list.extend(range(a, b+1))
    return num_list



import os
os.system('apt install xattr')
from xattr import *

def get_id(filename):
    """ Google Drive FileID """
    x = xattr(filename)
    file_id = x.get('user.drive.id')
    return file_id.decode()  # b'' -> str


import os
from IPython import get_ipython

# need this fix first
os.environ["LD_PRELOAD"] = ""
os.system("apt remove libtcmalloc-minimal4")
os.system("apt install libtcmalloc-minimal4")
os.environ["LD_PRELOAD"] = "/usr/lib/x86_64-linux-gnu/libtcmalloc_minimal.so.4.3.0"
os.system("dpkg -L libtcmalloc-minimal4")

# then install blender
url = "https://download.blender.org/release/Blender2.83/blender-2.83.0-linux64.tar.xz"
os.system(f"curl {url} | tar xJ")
os.system("ln -s /content/blender-2.83.0-linux64/blender /usr/local/bin/blender")
# show result
get_ipython().system("blender -v")


import os
import pandas as pd
from io import StringIO
from subprocess import getoutput
from IPython import get_ipython
import kora.data_table


# check if mounted, or explain
assert os.path.exists('/content/drive'), "You need to mount the drive first"
# check if kaggle.json exists, or explain
assert os.path.exists('/content/drive/My Drive/kaggle.json'), \
       "You need to create API token and store it as kaggle.json in your drive"

# copy API token to Colab
os.makedirs('/root/.kaggle', exist_ok=True)  # ~/.kaggle
os.system("cp 'drive/My Drive/kaggle.json' /root/.kaggle/")
os.chmod('/root/.kaggle/kaggle.json', 0o600)


def _show_csv(csv):
    """ Help display as table"""
    buf = StringIO(csv)
    if csv.startswith('Warning:'):
        buf.readline()
    return pd.read_csv(buf)

def search(query):
    """ Search for dataset names that match query """
    cmd = 'kaggle datasets list -v -s '+query
    return _show_csv(getoutput(cmd))

def ls(dataset):
    """ List all files for this dataset name """
    cmd = 'kaggle datasets files -v '+dataset
    return _show_csv(getoutput(cmd))

def download(dataset, only=None):
    """ Download all dataset files, or only a specific file """
    opt_f = ("-f "+only) if only else ""
    cmd = f"kaggle datasets download {dataset} --unzip {opt_f}"
    get_ipython().system(cmd)  # display too
    if opt_f:  # rename it back correctly
        for fn in os.listdir():
            if fn.startswith("datasets") and fn.endswith(only):
                os.rename(fn, only)



import sys
from pkg_resources import get_distribution

def get_ver(package):
    return get_distribution(package).version

__version__ = get_ver('kora')


IN_COLAB = 'google.colab' in sys.modules


# for %edit
if IN_COLAB:
    import kora.files



import os
import logging
from flask import Flask, request
from requests import post
from threading import Thread
os.system("pip install pyngrok")
from pyngrok import ngrok


log = logging.getLogger('werkzeug')  # Flask
log.setLevel(logging.ERROR)     # silence it


class Webhook:
    """ Line Webhook 
    
    Usage: 
    > webhook = Webhook(access_token)
    > webhook.start()   # then setup with this URL 
    > webhook.reply = lambda x: x   # just repeat
    """
    def __init__(self, access_token, port=5000):
        self.a_token = access_token
        self.port = port
        app = Flask("Line")   # to be webhook
        app.add_url_rule('/', None, self.serve, methods=['GET','POST'])
        app.add_url_rule('/shutdown', None, self.shutdown)
        self.app = app

    def shutdown(self):
        request.environ.get('werkzeug.server.shutdown')()
        return 'Server shutting down...'

    def serve(self):
        try:
            event = request.json['events'][0]
            token = event['replyToken']
            text = event['message']['text']
            self.send_reply(token, text)    
            return '{}', 200
        except:
            return "OK", 200

    def send_reply(self, token, text):
        url = 'https://api.line.me/v2/bot/message/reply'
        headers = {'Authorization': 'Bearer ' + self.a_token}
        data = {
            "replyToken": token,
            "messages":[{
                "type":"text",
                "text": self.reply(text) 
            }]
        }
        post(url, headers=headers, json=data)

    def reply(self, text):
        """ change this to what you like """
        return text

    def start(self, port=None):
        if port:
            self.port = port  # remember
        else:
            port = self.port
        url = ngrok.connect(port)
        self.url = url.replace('http', 'https')
        print(f"Webhook URL = {self.url}")
    
        self.thread = Thread(None, self.app.run, kwargs={"port":port})
        self.thread.start()

    def stop(self):
        """ must stop so many things"""
        ngrok.kill()
        os.system(f"curl 127.0.0.1:{self.port}/shutdown")
        self.thread.join()



"""
LIT : Language Interpretibility Toolkit
Help explain NLP models
"""

import os
import kora.install.yarn

# install dependencies
os.system("pip install lime sacrebleu transformers==2.11.0")

# download lit_nlp, and build front-end
os.chdir("/content")
os.system("npx degit PAIR-code/lit/lit_nlp#main lit_nlp")
os.chdir("/content/lit_nlp/client")
os.system("yarn && yarn build")
os.chdir("/content")



import os

# update apt
os.system("curl https://www.postgresql.org/media/keys/ACCC4CF8.asc | apt-key add -")
with open('/etc/apt/sources.list.d/pgdg.list', 'w') as f:
  f.write("deb http://apt.postgresql.org/pub/repos/apt/ bionic-pgdg main")
os.system("apt update")

# install PostgreSQL 13 and setup for root
os.system("apt install postgresql-13 postgresql-client-13")
os.system("service postgresql start")
os.system("sudo -u postgres psql -c 'CREATE USER root WITH SUPERUSER'")
os.system("psql postgres -c 'CREATE DATABASE root'")

# update %%sql and add pg special commands
os.system('pip install -U ipython-sql')
os.system('pip install pgspecial')
os.system('pip install psycopg2-binary')  # avoid warning

# config for %%sql
magic = get_ipython().run_line_magic
magic('load_ext', 'sql')
magic('config', 'SqlMagic.displaycon=False')
magic('config', 'SqlMagic.feedback=False')
magic('config', 'SqlMagic.autopandas=True')
magic('sql', 'postgresql+psycopg2://@/postgres')



"""
Yarn is used to develop front-end webapp
"""
import os

os.system('curl -sS https://dl.yarnpkg.com/debian/pubkey.gpg | apt-key add -')
os.system('echo "deb https://dl.yarnpkg.com/debian/ stable main" > /etc/apt/sources.list.d/yarn.list')
os.system('apt update')
os.system('apt install yarn')



""" Make bokeh easier to use in Colab
"""

from bokeh import *
from bokeh import io, plotting


def figure(height=600, width=600, **kw):
    """ Set height, width easily """
    kw['height'] = height
    kw['width'] = width
    return plotting.figure(**kw)


def _fig_repr_html(self):
    """ Figure can display itself """
    from bokeh.io.notebook import load_notebook
    from bokeh.embed.notebook import notebook_content

    load_notebook(hide_banner=True)
    (script, div, cell_doc) = notebook_content(self)
    return f'{div}<script>{script}</script>'

plotting.Figure._repr_html_ = _fig_repr_html



"""
RDKit is a library for Cheminformatics and Machine Learning
"""
import os

# Just extract from my custom tar. Very fast (1.7s)
url = 'https://github.com/airesearch-in-th/kora/releases/download/v0.6/rdkit.tar.gz'
os.system(f"curl -L {url} | tar xz -C /")



import os
from time import sleep

os.system("add-apt-repository -y ppa:dqlite/stable")
os.system("apt update")
os.system("apt install sqlite3")

print("Runtime is now restarting...")
print("You can ignore the error message [Your session crashed for an unknown reason.]")
sleep(0.5)
os._exit(0)  # restart



"""
My improvement to NLTK for Colab
"""
import os
from IPython.display import display, HTML

os.system("pip install nltk -U")  # 3.2.5 -> 3.5
from nltk import *                # replacement


def _print_concordance(self, word, width=80, lines=25):
    concordance_list = self.find_concordance(word, width=width)

    if not concordance_list:
        print("no matches")
    else:
        lines = min(lines, len(concordance_list))
        print("Displaying {} of {} matches:".format(lines, len(concordance_list)))
        html = "<table>\n"
        rep = f"<font color='green'>{word}</font>"
        for i, concordance_line in enumerate(concordance_list[:lines]):
            row = f"""
              <tr>
                <td>{concordance_line.left_print.replace(word, rep)}</td>
                <td>{rep}</td>
                <td>{concordance_line.right_print.replace(word, rep)}</td>
              </tr>
            """
            html += row
        html += "<style>td:first-child {text-align: right}</style>"
        display(HTML(html))

ConcordanceIndex.print_concordance = _print_concordance



import os
os.system('pip install py3Dmol')
import kora.install.rdkit

from rdkit import Chem
from rdkit.Chem import AllChem
import py3Dmol
from IPython.core.magic import register_line_magic


def plot(smiles, size=(200,200), style='stick'):
    """ 
    Visualize a molecule from its formula (smiles)
    Adapted from https://birdlet.github.io/2019/10/02/py3dmol_example 
    """
    mol = Chem.MolFromSmiles(smiles)
    mol = Chem.AddHs(mol)
    AllChem.EmbedMolecule(mol)
    AllChem.MMFFOptimizeMolecule(mol)
    block = Chem.MolToMolBlock(mol)

    viewer = py3Dmol.view(width=size[0], height=size[1])
    viewer.addModel(block, 'mol')
    viewer.setStyle({style:{}})
    viewer.zoomTo()
    viewer.show()


@register_line_magic
def ch(line):
  plot(line.strip(), size=(500,300))



"""
BEST dataset

For thai word segmentation
"""
import os
from urllib.request import urlretrieve


def download():
    base = 'http://www.nectec.or.th/corpuso/phocadownload/dl_text_thai-eng/BEST/TrainingSet/'
    genres = ['article', 'encyclopedia', 'news', 'novel']
    print("Start downloading: .. ")
    for genre in genres:
        url = base + genre + '.zip'
        fname, _ = urlretrieve(url)
        os.system(f'unzip {fname}')
        print(genre, end=' .. ')
    print('\nThe results are in these 4 folders')
    return genres



from json import *
import requests

from IPython.display import HTML

_render_template = """
<script src="https://rawgit.com/caldwell/renderjson/master/renderjson.js"></script>
<script>
renderjson.set_show_to_level(1)
document.body.appendChild(renderjson(%s))
new ResizeObserver(google.colab.output.resizeIframeToContent).observe(document.body)
</script>
"""

requests.models.Response._repr_html_ = lambda rsp: _render_template % rsp.text


def render(jstr):
    if type(jstr) != str:
        jstr = dumps(jstr)
    if jstr.startswith('http'):
        jstr = requests.get(jstr).text
    if jstr.endswith('.json'):
        with open(jstr) as f:
            jstr = f.read()
    return HTML(_render_template % jstr)



"""
TTC dataset

Thai Textbook Corpus(TTC) contains word list and frequencies.
"""
import os
import pandas as pd
from urllib.request import urlretrieve

def load_data(path='ttc_freq_extra.csv'):
    url = 'https://github.com/korakot/thainlp/raw/master/ttc_freq_extra.csv'
    if not os.path.exists(path):
        urlretrieve(url, path)
    return pd.read_csv(path)



import os
from IPython.core.magic import register_line_magic

os.system('wget -qO fd.deb https://github.com/sharkdp/fd/releases/download/v8.1.1/fd_8.1.1_amd64.deb')
os.system('dpkg -i fd.deb')
os.remove('fd.deb')

@register_line_magic
def fd(line):
    get_ipython().system('fd '+line)


""" Show elements in Venn Diagram """

from matplotlib_venn import venn3



def plot(A, B, U=['others'], names=['A', 'B', 'U']):
    """
    Show elements in 2 sets and optional others in U
    from https://stackoverflow.com/a/54604659/6729010
    """
    A = set(A)
    B = set(B)
    U = set(U) - A - B  # U can have all or only missing elements
    if len(names)==2:
        names.append('U')  # Add U if given only A, B 
    v = venn3([A,B,U], names)
    # set labels to be elements
    v.get_label_by_id('100').set_text(get_elem_str(A-B))
    v.get_label_by_id('110').set_text(get_elem_str(A&B))
    v.get_label_by_id('010').set_text(get_elem_str(B-A))
    v.get_label_by_id('001').set_text(get_elem_str(U))
    v.get_patch_by_id('001').set_color('white')
    return v


def get_elem_str(A: set):
    """ 1 column only, may use more in the futre """
    return '\n'.join(map(str,A))



import os
from matplotlib.animation import FuncAnimation

os.system("pip install bar_chart_race")
from bar_chart_race import *      # replaced
from bar_chart_race._make_chart import _BarChartRace


def anim(df, **kwargs):
    """ Show animation in Colab (with slider) """
    kw = dict(filename=None, orientation='h', sort='desc', n_bars=None, 
                   fixed_order=False, fixed_max=False, steps_per_period=10, 
                   period_length=500, interpolate_period=False, label_bars=True, 
                   bar_size=.95, period_label=True, period_fmt=None, 
                   period_summary_func=None, perpendicular_bar_func=None, figsize=(5, 3),
                   cmap=None, title=None, title_size=None, bar_label_size=7, 
                   tick_label_size=7, shared_fontdict=None, scale='linear', writer=None, 
                   fig=None, dpi=144, bar_kwargs=None, filter_column_colors=False)
    kw.update(kwargs)
    return _BarChartRace(df, **kw)

_BarChartRace._repr_html_ = lambda self: (
    FuncAnimation(self.fig, 
                  self.anim_func, 
                  range(len(self.df_values)), 
                  interval=self.period_length / self.steps_per_period
    ).to_jshtml() + 
    """<script>
      document.querySelector('.anim-state').state.value='once'
      document.querySelector('.fa-play').click() // autoplay not working?
      </script>
    """
)



"""
Taken from dataprep.eda. Modified to work in Colab.
"""
import os
os.system("pip install dataprep")
from dataprep.eda import *
from dataprep.eda import report


def _report_repr_html_(self):
    from bokeh.io.notebook import load_notebook
    from bokeh.embed.notebook import notebook_content
    load_notebook(hide_banner=True)
    script, div, _ = notebook_content(self.to_render)
    return f'{div}<script>{script}</script>'

report.Report._repr_html_ = _report_repr_html_



import os
os.system("pip install pyngrok")
from pyngrok import ngrok

# install anaconda3
os.system("wget -O ac.sh https://repo.anaconda.com/archive/Anaconda3-2020.07-Linux-x86_64.sh")
os.system("bash ./ac.sh -b")
os.remove("ac.sh")
os.symlink("/usr/local/lib/python3.6/dist-packages/google",
           "/root/anaconda3/lib/python3.8/site-packages/google")

# start jupyter
os.system("/root/anaconda3/bin/jupyter-lab --ip=0.0.0.0&")
# auto add path for new notebooks
startup_fname = "/root/.ipython/profile_default/startup/startup.py"
with open(startup_fname, 'w') as f:
  f.write("import os; os.environ['PATH'] = '/root/anaconda3/bin:' + os.environ['PATH']")

# show URL
url = ngrok.connect(8888)
print(url.replace('http','https')+'/tree')



# install chromium, its driver, and selenium
import os
os.system('apt update')
os.system('apt install chromium-chromedriver')
os.system('pip install selenium')
# set options to be headless, ..
from selenium import webdriver
options = webdriver.ChromeOptions()
options.add_argument('--headless')
options.add_argument('--no-sandbox')
options.add_argument('--disable-dev-shm-usage')
# create a webdriver instance, ready to use
wd = webdriver.Chrome('chromedriver',options=options)

# make it easier to query and explore elements
wd.select = wd.find_elements_by_css_selector
wd.select1 = wd.find_element_by_css_selector
from selenium.webdriver.remote.webelement import WebElement
WebElement.__str__ = lambda self: self.get_attribute('outerHTML')
WebElement.__repr__ = WebElement.__str__
WebElement.select = WebElement.find_elements_by_css_selector
WebElement.select1 = WebElement.find_element_by_css_selector
WebElement.__getitem__ = WebElement.get_attribute

# show screenshot easily with _repr_png_
def _screen_shot(self):
    from tempfile import NamedTemporaryFile as TempFile
    tmp = TempFile(suffix='.png')
    self.save_screenshot(tmp.name)
    return tmp.read()
webdriver.Chrome._repr_png_ = _screen_shot



import json
from altair import *
from altair import vega
from vega_datasets import data
from vega_datasets.core import Dataset

vega.Vega.renderers.enable('colab')  # allow vega display


# Adding data.thailand.url to Altair
data._datasets['thailand'] = 'thailand'

Dataset._dataset_info['thailand'] = {
    "filename": "thailand-topo.json",
    "format"  : "json",
    "is_local": False,
}

class Thailand(Dataset):
    """ Need to create a subclass to be searched and included 
    """
    name = "thailand"
    _return_type = dict
    
    def __init__(self, name):
        Dataset.__init__(self, name)
        self.url = 'https://raw.githubusercontent.com/korakot/thailand-hex-map/master/data/thailand-topo.json'

    def __call__(self, use_local=False, **kwargs):
        __doc__ = super(Thailand, self).__call__.__doc__  
        return json.loads(self.raw(use_local=use_local).decode(), **kwargs)



""" install neo4j, py2neo

Can be used as py2neo replacement. 
Use %%nj for easy query with Cypher
"""

import os
from IPython.core.magic import register_cell_magic


# download neo4j 4.1
print("Installing neo4j 4.1")
os.system("wget https://neo4j.com/artifact.php?name=neo4j-community-4.1.0-unix.tar.gz -O nj.tgz")
# decompress and rename
os.system("tar -xf nj.tgz")
os.system("mv neo4j-community-4.1.0 nj")
# download APOC plugin
os.system("wget -P nj/plugins https://github.com/neo4j-contrib/neo4j-apoc-procedures/releases/download/4.1.0.0/apoc-4.1.0.0-all.jar")
# config to disable password, use APOC
os.system("sed -i '/dbms.security.auth_enabled/s/^#//g' nj/conf/neo4j.conf")
os.system("sed -i '/dbms.security.procedures.unrestricted/s/^#//g' nj/conf/neo4j.conf")
os.system("sed -i 's/my.extensions.example,my.procedures/apoc/' nj/conf/neo4j.conf")
# start server
os.system("nj/bin/neo4j start")


# py2neo
print("Installing py2neo")
os.system("pip install py2neo")  # errors can be ignore
from py2neo import *   # e.g. Node, Relationship, Graph
graph = Graph("bolt://localhost:7687")  # connect to server


# Use %%nj similar to %%cypher
@register_cell_magic
def nj(line, cell=None):
    return graph.run(cell or line).to_data_frame()



import os
from IPython.core.magic import register_line_magic

os.system('wget -qO rg.deb https://github.com/BurntSushi/ripgrep/releases/download/12.1.1/ripgrep_12.1.1_amd64.deb')
os.system('dpkg -i rg.deb')
os.remove('rg.deb')

@register_line_magic
def rg(line):
    get_ipython().system('rg '+line)


import os
from pathlib import Path
os.system("pip install streamlit pyngrok")
from pyngrok import ngrok


def start(script=None):
    """ run the script and connect with ngrok"""
    if script is None:
        script = guess_file()
    url = ngrok.connect(8501)
    url = url.replace('http:', 'https:')
    print(url)
    os.system(f"streamlit run {script} --server.runOnSave=true &")


def stop():
    ngrok.kill()
    os.system("pkill streamlit")


def guess_file():
    for fn in ["main.py", "app.py", "hello.py"]:
        if Path(fn).exists():
            return fn
    # the first .py in current directory
    return next(Path().glob("*.py"))



""" Utility functions not available in google.colab.drive
    Mainly use pydrive
"""
import sys
import os
from os.path import basename
from pathlib import Path
from urllib.request import urlretrieve

from google.colab.drive import mount


def auth_drive():
    from pydrive.auth import GoogleAuth
    from pydrive.drive import GoogleDrive
    from oauth2client.client import GoogleCredentials as GC
    from google.colab import auth
    auth.authenticate_user()
    gauth = GoogleAuth()
    gauth.credentials = GC.get_application_default()
    return GoogleDrive(gauth)


def get_file(filename):
    """ Get this file if exists
    Or create a new one. Does not support folder, at root only.
    """
    drive = auth_drive()
    q = f"title = '{filename}' and 'root' in parents"
    res = drive.ListFile({'q': q}).GetList()
    if res:
        fid = res[0]['id']
        return drive.CreateFile({'id': fid})
    else:
        return drive.CreateFile({'title': filename})


def upload_public(filename):
    """ Upload the file to Google Drive and set permission to public """
    f = get_file(basename(filename))
    f.SetContentFile(filename)
    f.Upload()
    f.InsertPermission({'type': 'anyone', 'value': 'anyone', 'role': 'reader'})
    return 'https://drive.google.com/uc?id=' + f.get('id')  # direct link


def download_folder(folder_id):
    """ I copy this one from my old code, use drive API directly (not pydrive) """
    # authenticate
    from google.colab import auth
    auth.authenticate_user()
    # get folder_name
    from googleapiclient.discovery import build
    service = build('drive', 'v3')
    folder_name = service.files().get(fileId=folder_id).execute()['name']
    # install library 
    url = 'https://github.com/segnolin/google-drive-folder-downloader/raw/master/download.py'
    path = '/usr/local/lib/python3.6/dist-packages/download.py'
    urlretrieve(url, path)
    # recursive download
    import download
    download.download_folder(service, folder_id, './', folder_name)
    return folder_name


def get_path(file_id, drive=None):
    """ Use file_id to get the file path in Google Drive """
    if drive is None:
        drive = auth_drive()
    f = drive.CreateFile({'id': file_id})
    name = f['title']
    if f['parents']:
      parent_id = f['parents'][0]['id'] 
      return get_path(parent_id, drive) / name
    else:
      return Path(name)


def current_notebook():
    """ Get the filename, file_id of this notebook """
    import requests
    d = requests.get('http://172.28.0.2:9000/api/sessions').json()[0]
    file_id = d['path'].split('=')[1]
    return d['name'], file_id


def chdir_notebook():
    """ Change directory to its location in Google Drive"""
    # first confirm that drive is mounted
    if not Path("/content/drive").exists():
        mount("/content/drive")
    # then get the directory and change to it
    _, file_id = current_notebook()
    path = get_path(file_id)
    nb_dir = '/content/drive' / path.parent
    os.chdir(nb_dir)
    return nb_dir


def link_nbs():
    """ Create /nbs as a symlink to "Colab Notebooks".
    Also add /nbs to python path, so we can store libraries there
    Allow import A to get A.ipybn as well
    """
    if not Path("/content/drive").exists():
        mount("/content/drive")
    # create link if not already
    if not Path("/nbs").exists():
        os.symlink("/content/drive/My Drive/Colab Notebooks", "/nbs")
    # add path if not already
    if '/nbs' not in sys.path:
        sys.path.insert(5, '/nbs')  # before dist-packages
    # install import-ipynb, and set it to use /nbs
    os.system("pip install import-ipynb")
    import import_ipynb
    import_ipynb.find_notebook = find_in_nbs


def find_in_nbs(fullname, path=None):
    """ lookup a notebook file to import """
    nb_path = '/nbs/' + fullname + '.ipynb'
    if Path(nb_path).is_file():
        return nb_path
    # allow import Notebook_Name for Notebook Name.ipynb
    nb_path = nb_path.replace("_", " ")
    if Path(nb_path).is_file():
        return nb_path



import os

os.system("curl -fsSL https://bazel.build/bazel-release.pub.gpg | gpg --dearmor > bazel.gpg")
os.system("mv bazel.gpg /etc/apt/trusted.gpg.d/")
with open('/etc/apt/sources.list.d/bazel.list', 'w') as f:
    f.write("deb [arch=amd64] https://storage.googleapis.com/bazel-apt stable jdk1.8")
os.system("apt update && apt install bazel")



import os
from urllib.request import urlretrieve
url = "https://github.com/plotly/orca/releases/download/v1.2.1/orca-1.2.1-x86_64.AppImage"
orca = '/usr/local/bin/orca'
urlretrieve(url, orca)
os.chmod(orca, 0o755)
os.system("apt install xvfb libgconf-2-4")


import os

os.system('wget -qO libta.deb https://launchpad.net/~mario-mariomedina/+archive/ubuntu/talib/+files/libta-lib0_0.4.0-oneiric1_amd64.deb')
os.system('wget -qO ta.deb https://launchpad.net/~mario-mariomedina/+archive/ubuntu/talib/+files/ta-lib0-dev_0.4.0-oneiric1_amd64.deb')
os.system('dpkg -i libta.deb ta.deb')
os.system('rm libta.deb ta.deb')
os.system('pip install ta-lib')

from talib import *
# maybe I should move this talib.py to kora/talib.py instead
# people can then just use 'from kora import talib' to both install & import


import os
from importlib import import_module


def install(name, package_name=None):
    """ Try to import library, but may need to pip-install first 
    
    It's a lazy import/install. If it's already installed, calling this is fast.
    Try will just import it as normal.
    """
    try:
        return import_module(name)
    except:
        if package_name is None:  # could be different too
            package_name = name  
        os.system("pip install " + package_name)
        return import_module(name)



from google.colab.output import *
from google.colab.output._js_builder import Js
from IPython.display import display, Javascript
from typing import Union

# path that can serve content
nbx = '/usr/local/share/jupyter/nbextensions'

def set_height(h: Union[str, int]):
    eval_js('google.colab.output.setIframeHeight("%s")' % h)

# Usage:
# set_style(fontSize='30px', color='red')
# print("Hello")
def set_style(**kw):
    import json
    display(Javascript('''
    for (rule of document.styleSheets[0].cssRules){
        if (rule.selectorText=='body') {
            Object.assign(rule.style, %s)
            break
        }
    }
    ''' % json.dumps(kw)))


def url_port(port: int):
    """ Convert port number to URL for webapps """
    return eval_js("google.colab.kernel.proxyPort(%d)" % port)


def show_port(port, height=400):
    """ Display webapp at port number in output using iframe """
    display(Javascript("""
    (async ()=>{
        fm = document.createElement('iframe')
        fm.src = await google.colab.kernel.proxyPort(%s)
        fm.width = '95%%'
        fm.height = '%d'
        fm.frameBorder = 0
        document.body.append(fm)
    })();
    """ % (port, height) ))


def _js_getattr(self, name):
    val = self._attr_map.get(name, None)
    if val is None:
        val = self._builder(self._join(self._js_value(), name))
        self._attr_map[name] = val
    # if it's a common attribute, just eval() it automatically
    if name in {'name', 'value', 'id', 'length', 'size', 
                'textContent', 'innerText','innerHTML', 'outerHTML'}:
        return val.eval()
    else:
        return val

Js.__getattr__ = _js_getattr



import os
from IPython.display import display, Javascript, Audio
from google.colab import output
from base64 import b64decode


RECORD_AUDIO = """
const sleep  = time => new Promise(resolve => setTimeout(resolve, time))
const b2text = blob => new Promise(resolve => {
    const reader = new FileReader()
    reader.onloadend = e => resolve(e.srcElement.result)
    reader.readAsDataURL(blob)
})
var record = time => new Promise(async resolve => {
    stream = await navigator.mediaDevices.getUserMedia({ audio: true })
    recorder = new MediaRecorder(stream)
    chunks = []
    recorder.ondataavailable = e => chunks.push(e.data)
    recorder.start()
    await sleep(time)
    recorder.onstop = async ()=>{
        blob = new Blob(chunks)
        text = await b2text(blob)
        resolve(text)
    }
    recorder.stop()
})
"""

def audio(sec=3):
    """ Record audio for a given duration """
    display(Javascript(RECORD_AUDIO))
    s = output.eval_js('record(%d)' % (sec*1000))
    b = b64decode(s.split(',')[1])
    return Audio(b)


# Add to class Audio
def _audio_save(self, filename=None):
    """ Audio's method to save as a file 
        Default format is probably webm
    """
    with open('/tmp/audio.webm', 'wb') as f:
        f.write(self.data)
    os.system("ffmpeg -y -i /tmp/audio.webm "+filename)
    return filename
Audio.save = _audio_save


"""
Allow using normal Jupyter Notebook and Jupyter Lab in Colab
"""

import os
os.system("pip install jupyterlab")
os.system("pip install pyngrok")
from pyngrok import ngrok


def start(lab=False):
    url = ngrok.connect(port=8888)
    url = url.replace('http://','https://')
    if not lab:
        url += '/tree'  # normal jupyter notebook
    os.system("jupyter lab --ip=0.0.0.0&")
    print(url)


def stop():
    ngrok.kill()
    os.system("pkill jupyter-lab")



"""
SCB-MT-EN-TH

SCB Machine Translation Dataset
More than 1 millions sentence pairs
"""

import os

def download():
    url = "https://github.com/vistec-AI/dataset-releases/releases/download/scb-mt-en-th-2020_v1.0/scb-mt-en-th-2020.zip"
    print("Start downloading: .. ")
    os.system(f"wget {url}")
    os.system(f"unzip -j scb-mt-en-th-2020.zip")
    print("""\
  There are 12 files:
    task_master_1.csv                 Chat dialog to achieve a task
    generated_reviews_translator.csv  Reviews translated by translators
    generated_reviews_yn.csv          Reviews checked by translators
    generated_reviews_crowd.csv	      Reviews translated by crowd-sourcing
    nus_sms.csv                       SMS messages (crowd-source)
    msr_paraphrase.csv                Microsoft paraphrase (crowd-source)
    mozilla_common_voice.csv          Mozilla Common Voices ASR (crowd-source)
    assorted_government.csv           Bilingual government documents
    thai_websites.csv                 Bilingual Top-500 Thai websites (crawling)  
    paracrawl.csv                     Selections from Paracrawl
    wikipedia.csv                     Selections from Wikipedia
    apdf.csv                          Asia Pacific Defence Forum""")



""" What bs4 should have been """

from bs4 import BeautifulSoup
import requests 
from os.path import splitext
from pathlib import Path


def Soup(s, features='lxml', **kw):
    """ A fake class, of what BeautifulSoup should have been 
    
    It accepts a url or a file, in addition to html/xml as usual
    """
    if isinstance(s, Path):
        src = s.read_text()
    elif s.startswith('http'):
        src = requests.get(s).text
    elif splitext(s)[1] in ('.html', '.xml'):
        src = open(s).read()
    else:
        src = s
    return BeautifulSoup(src, features, **kw)
